@@ -128,6 +128,10 @@ def main():
     help='Updates the source (and/or provider if applicable)',
     show_default=True
     )
+@click.option('--upgrade', '-U', is_flag=True, default=False,
+    help='Installs the latest provider versions (init only)',
+    show_default=True
+    )
 @click.option('--planfile', '-p', is_flag=True, default=False,
     help='Use the generated planfile when applying the changes',
     show_default=True
@@ -136,26 +140,32 @@ def main():
     help='Do not ask for confirmation before applying planned changes (where applicable)',
     show_default=True
     )
+@click.option('--clean', '-c', is_flag=True, default=False,
+    help='Clean up .terragrunt-cache before executing the command',
+    show_default=True
+    )
 @click.option('--working-dir', '-w', default=None,
     help='Working directory, when omitted the current directory is used',
     )
 @click.argument('terragrunt-args', nargs=-1, type=click.UNPROCESSED)
 @click.version_option(version=__version__)
-def run(command, verbose, debug, dry_run, no_lock, update,
-    planfile, auto_approve, working_dir, terragrunt_args):
+def run(command, verbose, debug, dry_run, no_lock, update, upgrade,
+    planfile, auto_approve, clean, working_dir, terragrunt_args):
     """ [default] Executes a terragrunt command on a single project """
 
     check_latest_version(verbose)
 
-    tgwrap = TgWrap(verbose=verbose)
+    tgwrap = TgWrap(verbose=verbose, check_tg_source=True)
     tgwrap.run(
         command=command,
         debug=debug,
         dry_run=dry_run,
         no_lock=no_lock,
         update=update,
+        upgrade=upgrade,
         planfile=planfile,
         auto_approve=auto_approve,
+        clean=clean,
         working_dir=working_dir,
         terragrunt_args=terragrunt_args,
     )
@@ -187,6 +197,10 @@ def run(command, verbose, debug, dry_run, no_lock, update,
     help='Updates the source (and/or provider if applicable)',
     show_default=True
     )
+@click.option('--upgrade', '-U', is_flag=True, default=False,
+    help='Installs the latest provider versions (init only)',
+    show_default=True
+    )
 @click.option('--exclude-external-dependencies/--include-external-dependencies', '-x/-i',
     is_flag=True, default=True,
     help='Whether or not external dependencies must be ignored',
@@ -204,6 +218,10 @@ def run(command, verbose, debug, dry_run, no_lock, update,
     help='Do not ask for confirmation before applying planned changes (where applicable)',
     show_default=True
     )
+@click.option('--clean', '-c', is_flag=True, default=False,
+    help='Clean up .terragrunt-cache before executing the command',
+    show_default=True
+    )
 @click.option('--working-dir', '-w', default=None,
     help='Working directory, when omitted the current directory is used',
     )
@@ -215,39 +233,41 @@ def run(command, verbose, debug, dry_run, no_lock, update,
     )
 @click.option('--include-dir', '-I',
     multiple=True, default=[],
-    help='A glob of a directory that needs to be included, this option can be used multiple times',
+    help='A glob of a directory that needs to be included, this option can be used multiple times. For example: -I "integrations/\*/\*"',
     show_default=True
     )
 @click.option('--exclude-dir', '-E',
     multiple=True, default=[],
-    help='A glob of a directory that needs to be excluded, this option can be used multiple times',
-    show_default=True
+    help='A glob of a directory that needs to be excluded, this option can be used multiple times. For example: -E "integrations/\*/\*"',
+    show_default=True,
     )
 @click.argument('terragrunt-args', nargs=-1, type=click.UNPROCESSED)
 @click.version_option(version=__version__)
-def run_all(command, verbose, debug, dry_run, no_lock, update, exclude_external_dependencies,
-    step_by_step, planfile, auto_approve, working_dir, start_at_step,
+def run_all(command, verbose, debug, dry_run, no_lock, update, upgrade, exclude_external_dependencies,
+    step_by_step, planfile, auto_approve, clean, working_dir, start_at_step,
     limit_parallelism, include_dir, exclude_dir, terragrunt_args):
     """ Executes a terragrunt command across multiple projects """
 
     check_latest_version(verbose)
 
-    tgwrap = TgWrap(verbose=verbose)
+    tgwrap = TgWrap(verbose=verbose, check_tg_source=True)
     tgwrap.run_all(
         command=command,
         debug=debug,
         dry_run=dry_run,
         no_lock=no_lock,
         update=update,
+        upgrade=upgrade,
         exclude_external_dependencies=exclude_external_dependencies,
         step_by_step=step_by_step,
         planfile=planfile,
         auto_approve=auto_approve,
+        clean=clean,
         working_dir=working_dir,
         start_at_step=start_at_step,
         limit_parallelism=limit_parallelism,
-        include_dir=include_dir,
-        exclude_dir=exclude_dir,
+        include_dirs=include_dir,
+        exclude_dirs=exclude_dir,
         terragrunt_args=terragrunt_args,
     )
 
@@ -305,10 +325,6 @@ def run_import(address, id, verbose, dry_run, working_dir, no_lock, terragrunt_a
     help='Verbose printing',
     show_default=True
     )
-@click.option('--dry-run', '-D', is_flag=True, default=False,
-    help='dry-run mode, no real actions are executed (only in combination with step-by-step mode)',
-    show_default=True
-    )
 @click.option('--exclude-external-dependencies/--include-external-dependencies', '-x/-i',
     is_flag=True, default=True,
     help='Whether or not external dependencies must be ignored',
@@ -324,30 +340,47 @@ def run_import(address, id, verbose, dry_run, working_dir, no_lock, terragrunt_a
     help='Show output as json',
     show_default=True
     )
-@click.option('--terrasafe-config', '-T', default=None,
-    help='Name of the terrasafe config file (or set the TERRASAFE_CONFIG environment variable)',
-    envvar='TERRASAFE_CONFIG', type=click.Path(),
+@click.option('--analyze-config', '-A', default=None,
+    help='Name of the analyze config file (or set TGWRAP_ANALYZE_CONFIG environment variable)',
+    envvar='TGWRAP_ANALYZE_CONFIG', type=click.Path(),
     )
 @click.option('--parallel-execution', '-p', is_flag=True, default=False,
     help='Whether or not to use parallel execution',
     )
+@click.option('--include-dir', '-I',
+    multiple=True, default=[],
+    help='A glob of a directory that needs to be included, this option can be used multiple times. For example: -I "integrations/\*/\*"',
+    show_default=True
+    )
+@click.option('--exclude-dir', '-E',
+    multiple=True, default=[],
+    help='A glob of a directory that needs to be excluded, this option can be used multiple times. For example: -E "integrations/\*/\*"',
+    show_default=True,
+    )
+@click.option('--planfile-dir', '-P', default=None,
+    help='Relative path to directory with plan file (or set TGWRAP_PLANFILE_DIR environment variable), see README for more details',
+    envvar='TGWRAP_PLANFILE_DIR', type=click.Path(),
+    )
 @click.argument('terragrunt-args', nargs=-1, type=click.UNPROCESSED)
 @click.version_option(version=__version__)
-def analyze(verbose, dry_run, exclude_external_dependencies, working_dir, 
-    start_at_step, out, parallel_execution, terrasafe_config, terragrunt_args):
+def run_analyze(verbose, exclude_external_dependencies, working_dir, start_at_step,
+            out, analyze_config, parallel_execution,
+            include_dir, exclude_dir, planfile_dir, terragrunt_args):
     """ Analyzes the plan files """
 
     check_latest_version(verbose)
 
     tgwrap = TgWrap(verbose=verbose)
     tgwrap.analyze(
-        dry_run=dry_run,
         exclude_external_dependencies=exclude_external_dependencies,
         working_dir=working_dir,
         start_at_step=start_at_step,
         out=out,
+        analyze_config=analyze_config,
         parallel_execution=parallel_execution,
-        terrasafe_config=terrasafe_config,
+        include_dirs=include_dir,
+        exclude_dirs=exclude_dir,
+        planfile_dir=planfile_dir,
         terragrunt_args=terragrunt_args,
     )
 
@@ -382,7 +415,7 @@ def lock(module, auto_approve, verbose, dry_run, working_dir):
 
     check_latest_version(verbose)
 
-    tgwrap = TgWrap(verbose=verbose)
+    tgwrap = TgWrap(verbose=verbose, check_tg_source=True)
     tgwrap.set_lock(
         module=module,
         lock_status="lock",
@@ -422,7 +455,7 @@ def unlock(module, auto_approve, verbose, dry_run, working_dir):
 
     check_latest_version(verbose)
 
-    tgwrap = TgWrap(verbose=verbose)
+    tgwrap = TgWrap(verbose=verbose, check_tg_source=True)
     tgwrap.set_lock(
         module=module,
         lock_status="unlock",
@@ -454,7 +487,7 @@ def unlock(module, auto_approve, verbose, dry_run, working_dir):
 @click.option('--module', '-m', default="",
     help='Name of the module, if omitted all modules will be copied.',
     )
-@click.option('--clean-up', '-c', is_flag=True, default=False,
+@click.option('--clean', '-c', is_flag=True, default=False,
     help='Clean up files on target side that do not exist as source',
     show_default=True
     )
@@ -480,7 +513,7 @@ def unlock(module, auto_approve, verbose, dry_run, working_dir):
 @click.version_option(version=__version__)
 def sync(
     source_domain, source_stage, target_domain, target_stage, module, auto_approve,
-    verbose, dry_run, clean_up, include_lock_file, working_dir
+    verbose, dry_run, clean, include_lock_file, working_dir
     ):
     """ Syncs the terragrunt config files from one stage to another (and possibly to a different domain) """
 
@@ -495,7 +528,7 @@ def sync(
         module=module,
         auto_approve=auto_approve,
         dry_run=dry_run,
-        clean_up=clean_up,
+        clean=clean,
         include_lock_file=include_lock_file,
         working_dir=working_dir,
     )
@@ -512,7 +545,7 @@ def sync(
 @click.option('--target-directory', '-t', required=True,
     help='Directory where config files will be synced to.',
     )
-@click.option('--clean-up', '-c', is_flag=True, default=False,
+@click.option('--clean', '-c', is_flag=True, default=False,
     help='Clean up files on target side that do not exist as source',
     show_default=True
     )
@@ -538,7 +571,7 @@ def sync(
 @click.version_option(version=__version__)
 def sync_dir(
     source_directory, target_directory, auto_approve,
-    verbose, dry_run, clean_up, include_lock_file, working_dir
+    verbose, dry_run, clean, include_lock_file, working_dir
     ):
     """ Syncs the terragrunt config files from one directory to anothery """
 
@@ -550,7 +583,7 @@ def sync_dir(
         target_directory=target_directory,
         auto_approve=auto_approve,
         dry_run=dry_run,
-        clean_up=clean_up,
+        clean=clean,
         include_lock_file=include_lock_file,
         working_dir=working_dir,
     )
@@ -564,15 +597,21 @@ def sync_dir(
 @click.option('--manifest-file', '-m',
     help='Manifest file describing the deployment options',
     required=True, default="manifest.yaml", show_default=True,
+    type=click.Path(),
     )
 @click.option('--version-tag', '-V',
     help="Version tag, use 'latest' to, well, get the latest version.",
     required=True, default='latest', show_default=True,
     )
 @click.option('--target-stage', '-t',
+    multiple=True,
     help='Stage to deploy to',
     type=click.Choice(STAGES, case_sensitive=True), required=True,
     )
+@click.option('--include-global-config-files/--exclude-global-config-files', '-i/-x',
+    help='Whether or not to include deploying the (in the manifest specified) global config files',
+    is_flag=True, default=True, show_default=True,
+    )
 @click.option('--auto-approve', '-a',
     help='Do not ask for confirmation before applying planned changes (where applicable)',
     is_flag=True, default=False, show_default=True,
@@ -590,10 +629,10 @@ def sync_dir(
     )
 @click.version_option(version=__version__)
 def deploy(
-    manifest_file, version_tag, target_stage,
+    manifest_file, version_tag, target_stage, include_global_config_files,
     auto_approve, verbose, dry_run, working_dir
     ):
-    """ [experimental] Deploys the terragrunt config files from a git repository """
+    """ Deploys the terragrunt config files from a git repository """
 
     check_latest_version(verbose)
 
@@ -601,12 +640,48 @@ def deploy(
     tgwrap.deploy(
         manifest_file=manifest_file,
         version_tag=version_tag,
-        target_stage=target_stage,
+        target_stages=target_stage,
+        include_global_config_files=include_global_config_files,
         auto_approve=auto_approve,
         dry_run=dry_run,
         working_dir=working_dir,
     )
 
+@main.command(
+    name="check-deployments",
+    context_settings=dict(
+        ignore_unknown_options=True,
+    ),
+)
+@click.option('--manifest-file', '-m',
+    help='Manifest file describing the deployment options',
+    required=True, default="manifest.yaml", show_default=True,
+    type=click.Path(),
+    )
+@click.option('--verbose', '-v',
+    help='Verbose printing',
+    is_flag=True, default=False, show_default=True,
+    )
+@click.option('--working-dir', '-w', default=None,
+    help='Working directory, when omitted the current directory is used',
+    )
+@click.option('--out', '-o', is_flag=True, default=False,
+    help='Show output as json',
+    show_default=True
+    )
+@click.version_option(version=__version__)
+def check_deployments(manifest_file, verbose, working_dir, out):
+    """ Check the freshness of deployed configuration versions against the platform repository """
+
+    check_latest_version(verbose)
+
+    tgwrap = TgWrap(verbose=verbose)
+    tgwrap.check_deployments(
+        manifest_file=manifest_file,
+        working_dir=working_dir,
+        out=out,
+    )
+
 @main.command(
     name="show-graph",
     context_settings=dict(
@@ -629,13 +704,13 @@ def deploy(
     )
 @click.option('--include-dir', '-I',
     multiple=True, default=[],
-    help='A glob of a directory that needs to be included, this option can be used multiple times',
+    help='A glob of a directory that needs to be included, this option can be used multiple times. For example: -I "integrations/\*/\*"',
     show_default=True
     )
 @click.option('--exclude-dir', '-E',
     multiple=True, default=[],
-    help='A glob of a directory that needs to be excluded, this option can be used multiple times',
-    show_default=True
+    help='A glob of a directory that needs to be excluded, this option can be used multiple times. For example: -E "integrations/\*/\*"',
+    show_default=True,
     )
 @click.option('--working-dir', '-w', default=None,
     help='Working directory, when omitted the current directory is used',
@@ -647,13 +722,13 @@ def show_graph(verbose, backwards, exclude_external_dependencies, working_dir, i
 
     check_latest_version(verbose)
 
-    tgwrap = TgWrap(verbose=verbose)
+    tgwrap = TgWrap(verbose=verbose, check_tg_source=True)
     tgwrap.show_graph(
         backwards=backwards,
         exclude_external_dependencies=exclude_external_dependencies,
         working_dir=working_dir,
-        include_dir=include_dir,
-        exclude_dir=exclude_dir,
+        include_dirs=include_dir,
+        exclude_dirs=exclude_dir,
         terragrunt_args=terragrunt_args,
     )
 
@@ -681,3 +756,41 @@ def clean(verbose, working_dir):
         working_dir=working_dir,
     )
 
+@main.command(
+    name="change-log",
+    context_settings=dict(
+        ignore_unknown_options=True,
+    ),
+)
+@click.option('--changelog-file', '-c',
+    help='Existing change log file, if passed the content will be included in the given file',
+    required=False,
+    type=click.Path(),
+    )
+@click.option('--verbose', '-v', is_flag=True, default=False,
+    help='Verbose printing',
+    show_default=True
+    )
+@click.option('--working-dir', '-w', default=None,
+    help='Working directory, when omitted the current directory is used',
+    )
+@click.option('--include-nbr-of-releases', '-i',
+    type=int, default=20, show_default=True,
+    help='Max number of releases to include',
+    )
+@click.version_option(version=__version__)
+def change_log(changelog_file, verbose, working_dir, include_nbr_of_releases):
+    """ Experimental! Generate a change log """
+
+    check_latest_version(verbose)
+
+    tgwrap = TgWrap(verbose=verbose)
+    tgwrap.change_log(
+        changelog_file=changelog_file,
+        working_dir=working_dir,
+        include_nbr_of_releases=include_nbr_of_releases,
+    )
+
+# this is needed for the vscode debugger to work
+if __name__ == '__main__':
+    main()
